-- create-todo.sql
--


-- create object_type "todo_item"

select acs_object_type__create_type ('todo_item', 'To Do Item', 'To Do Items', 'acs_object', 'TODO_ITEMS', 'ITEM_ID', 'todo_item', 'f', null, 'todo_item__name' );

-- declare the attributes

select acs_attribute__create_attribute ('todo_item','description','text','description','descriptions',null,'description',null,1,1,null,'type_specific','f');

select acs_attribute__create_attribute ('todo_item','status','string','status','status',null,'status',null,1,1,null,'type_specific','f');

select acs_attribute__create_attribute ('todo_item','owner_id','integer','owner','owners',null,'owner_id',null,1,1,null,'type_specific','f');
    
select acs_attribute__create_attribute ('todo_item','due_date','date','due date','due dates',null,'due_date','now()',1,1,null,'type_specific','f');


-- Package todo_item

-- new()
select define_function_args('todo_item__new','item_id,title,description,status,due_date,owner_id,package_id,context_id,creation_user,creation_ip');

CREATE OR REPLACE FUNCTION todo_item__new (integer,varchar,text,char,varchar,integer,integer,integer,integer,varchar) RETURNS integer AS '
DECLARE
        p_item_id       ALIAS FOR $1;
        p_title         ALIAS FOR $2;
        p_description   ALIAS FOR $3;
        p_status        ALIAS FOR $4;
        p_due_date      ALIAS FOR $5;
        p_owner_id      ALIAS FOR $6;
        p_package_id    ALIAS FOR $7;
        p_context_id    ALIAS FOR $8;
        p_creation_user ALIAS FOR $9;
        p_creation_ip   ALIAS FOR $10;
                
        v_id integer;
        v_type varchar;
        v_due_date timestamptz;
BEGIN
        
        v_type := ''todo_item'';
        
        v_id := acs_object__new(
            p_item_id,
            v_type,
            now(),
            p_creation_user,
            p_creation_ip,
            p_context_id::integer,
            p_title,
            p_package_id
        );

        if p_due_date is null then
          v_due_date := now();
        else
          select to_date(p_due_date,''YYYY MM DD'') into v_due_date from dual;
        end if;

   insert into todo_items
          (item_id, description, status, due_date, owner_id)
   values
          (p_item_id, p_description, p_status, v_due_date , p_owner_id);
   return v_id;

END;
' LANGUAGE 'plpgsql';

-- delete()
select define_function_args('todo_item__delete','item_id');

CREATE OR REPLACE FUNCTION todo_item__delete (integer) RETURNS integer AS '
DECLARE
        p_item_id ALIAS FOR $1;        
BEGIN        
   delete from todo_items where item_id = p_item_id;
   PERFORM acs_object__delete(p_item_id);
   return 0;
END;
' LANGUAGE 'plpgsql';

-- name()
select define_function_args('todo_item__name','item_id');

create or replace function todo_item__name(integer)
returns varchar as '
declare
    p_item_id                      alias for $1;
begin
    return acs_object__name(p_item_id);
end;
' language 'plpgsql';


-- update()
select define_function_args('todo_item__update','item_id,title,description,due_date,status,user_id');

create or replace function todo_item__update(integer,varchar,text,varchar,char,integer)
returns integer as '
DECLARE
        p_item_id       ALIAS FOR $1;
        p_title         ALIAS FOR $2;
        p_description   ALIAS FOR $3;
        p_due_date      ALIAS FOR $4;
        p_status        ALIAS FOR $5;
        p_user_id       ALIAS FOR $6;
                
        v_due_date timestamptz;
BEGIN
        
        if p_due_date is null then
          v_due_date := now();
        else
          select to_date(p_due_date,''YYYY MM DD'') into v_due_date from dual;
        end if;

        update todo_items
               set description = p_description,
                   status = p_status,
                   due_date = v_due_date
               where item_id = p_item_id;

        update acs_objects set title = p_title where object_id = p_item_id;

        return acs_object__update_last_modified(p_item_id,p_user_id,null,now());

END;
' language 'plpgsql';


-- Create table for todo_items

create table todo_items (
    item_id integer,
    description text,
    status char(1),
    owner_id integer,
    due_date date default now(),
    constraint todo_items_item_id_pk primary key (item_id),
    constraint todo_items_owner_id_fk foreign key (owner_id) references users,
    constraint todo_items_item_id_fk foreign key (item_id) references acs_objects
);

create view todo_items_all AS select * from todo_items join acs_objects on (item_id = object_id);
